const initializeDatabase = require('../../db/initialize');
const UsersDBApi = require("../../db/api/users");
const bcrypt = require("bcrypt");
const config = require("../config");

const ValidationError = require("./notifications/errors/validation");
const EmailSender = require("./email");
const WatiService = require("../services/wati");

module.exports = class UsersService {
  static async create(data, currentUser, sendInvitationEmails = true) {
    const db = await initializeDatabase(); 
    const transaction = await db.sequelize.transaction(); 
    let email = data.email;
    let emailsToInvite = [];
    try {
      if (email) {
        let user = await UsersDBApi.findBy({ email }, { transaction });
        if (user) {
          throw new ValidationError("iam.errors.userAlreadyExists");
        } else {
          await UsersDBApi.create(
            { data },
            {
              currentUser,
              transaction,
            }
          );
          emailsToInvite.push(email);
          const mailSender = new EmailSender();
          try {
            await mailSender.sendEmail({
              dynamicData: {},
              mailProps: { to: mailSender.adminMail },
              template: mailSender.htmlTemplates.NEW_USER__ADMIN,
            });
            // await mailSender.sendEmail({
            //   dynamicData: {},
            //   mailProps: { to: data.email },
            //   template:mailSender.htmlTemplates.NEW_USER__USER
            // });

            const watiObj = new WatiService();
            WatiService.sendWhatsappMessage(
              data.phoneNumber,
              watiObj.templates.userCreated
            );
            // WatiService.sendWhatsappMessage(
            //   watiObj.adminPhone,
            //   watiObj.templates.userCreatedAdmin,
            //   [{
            //     "name": "user_name",
            //     "value": `${data.firstName} ${data.lastName}`
            //   }]
            // )
          } catch (error) {
            console.log("error", error);
          }
        }
      }
      await transaction.commit();
    } catch (error) {
      await transaction.rollback();
      throw error;
    }
    if (emailsToInvite && emailsToInvite.length) {
      if (!sendInvitationEmails) {
        return;
      }
      // AuthService.sendPasswordResetEmail(email, 'invitation', host);
    }
  }

  static async update(data, id, currentUser) {
    const db = await initializeDatabase();
    let transaction;
    
    try {
      // Get sequelize instance from db and start transaction
      transaction = await db.sequelize.transaction();
      
      // Check if user exists
      const users = await UsersDBApi.findBy(
        { id }, 
        { transaction }
      );
  
      if (!users) {
        throw new ValidationError("User not found");
      }
  
      // Handle password update
      if (data.password) {
        data.password = bcrypt.hashSync(
          data.password,
          config.bcrypt.saltRounds
        );
      } else {
        data.password = users.password;
      }
  
      // Update the user
      await UsersDBApi.update(id, data, {
        currentUser,
        transaction 
      });
  
      // Commit transaction
      await transaction.commit();
      
      return users;
  
    } catch (error) {
      // Rollback transaction on error
      if (transaction) await transaction.rollback();
      throw new ValidationError(error.message);
    }
  }

  static async remove(id, currentUser) {
    const db = await initializeDatabase();
    const transaction = await db.sequelize.transaction();

    try {
      if (currentUser.id === id) {
        throw new ValidationError("iam.errors.deletingHimself");
      }

      if (currentUser.role !== "admin") {
        throw new ValidationError("errors.forbidden.message");
      }

      await UsersDBApi.remove(id, {
        currentUser,
        transaction,
      });

      await transaction.commit();
      return { message: "success" };
    } catch (error) {
      await transaction.rollback();
      throw error;
    }
  }

  static async getUserDetails(id) {
      const result = await UsersDBApi.findBy({ id: id });
      console.log("result --- booking", result);
      return result;
  }
};
