import {
  Body,
  Controller,
  Get,
  Logger,
  Param,
  ParseIntPipe,
  Post,
  Put,
  Query,
  BadRequestException,
} from '@nestjs/common';
import {
  ApiBody,
  ApiCreatedResponse,
  ApiOkResponse,
  ApiOperation,
} from '@nestjs/swagger';
import { sendResponseMessage } from 'src/common/common-utils';
import { APIFailedException } from 'src/common/exceptions';
import { Bookings } from 'src/entities';
import { BookingsService } from './bookings.service';
import {
  AssignedBookingQuery,
  BookingDetailsQuery,
  CreateBookingPayload,
  CreateBookingsDto,
  EditBookingPayload,
  responseMessage,
  UpdateBookingPayload,
  UpdateBookingResponse,
  UpdateBookingStatuses,
} from './models/createBooking.dto';
import { GetBookingsDto } from './models/getBookings.dto';

interface ServiceResponse<T> {
  status: string;
  data: T;
}
interface BookingResponse extends ServiceResponse<Bookings> {
  data: Bookings;
}

interface BookingsArrayResponse extends ServiceResponse<Bookings[]> {
  data: Bookings[];
}

interface UpdatePujaStatusResponse {
  status: string;
  data: UpdateBookingResponse;
}

interface BookingDetailsResponse {
  status: string;
  data: Bookings;
}

interface BookingStatusResponse {
  status: string;
  message: string;
}

@Controller('booking')
export class BookingsController {
  logger: Logger;

  constructor(private readonly bookingsService: BookingsService) {
    this.logger = new Logger();
  }

  @Post('pujari-bookings')
  @ApiOperation({
    summary: 'Get the list of bookings of pujari based on the selected pujari',
  })
  @ApiOkResponse({ type: CreateBookingsDto, isArray: true })
  async bookings(@Body() body: GetBookingsDto): Promise<BookingsArrayResponse> {
    try {
      const result = await this.bookingsService.getBookings(body);

      if (!result || !Array.isArray(result)) {
        throw new APIFailedException('Invalid booking data received');
      }

      return {
        status: 'success',
        data: result,
      };
    } catch (error) {
      this.logger.error('Failed to get bookings:', error);
      throw new APIFailedException(error);
    }
  }

  @Get('pujari-assigned-bookings')
  @ApiOperation({
    summary: 'Get the list of pujari assigned bookings',
  })
  @ApiOkResponse({ type: CreateBookingsDto, isArray: true })
  async assign(
    @Query() query: AssignedBookingQuery
  ): Promise<BookingsArrayResponse> {
    try {
      const result = await this.bookingsService.getAssignedBookingstoPujari(
        query
      );

      if (!result || !Array.isArray(result)) {
        throw new APIFailedException('Invalid assigned bookings data');
      }

      return {
        status: 'success',
        data: result,
      };
    } catch (error) {
      this.logger.error('Failed to get assigned bookings:', error);
      throw new APIFailedException(error);
    }
  }

  @Get('assigned-bookings')
  @ApiOperation({
    summary: 'Get the list of assigned bookings',
  })
  @ApiOkResponse({ type: CreateBookingsDto, isArray: true })
  async getAssignedBookings(
    @Query() query: AssignedBookingQuery
  ): Promise<BookingsArrayResponse> {
    try {
      const bookings = await this.bookingsService.getAssignedBookings(query);
      if (!bookings) {
        throw new APIFailedException('No bookings found');
      }
      return {
        status: 'success',
        data: bookings,
      };
    } catch (error) {
      this.logger.error('Failed to get assigned bookings:', error);
      throw new APIFailedException(error);
    }
  }

  @Post('puja-by-bookingId/:bookingId')
  @ApiOperation({
    summary: 'Get specific puja by unique ID',
  })
  @ApiOkResponse({ type: CreateBookingsDto })
  async getPujaByUniqueId(
    @Param('bookingId') bookingId: string
  ): Promise<BookingResponse> {
    try {
      if (!bookingId) {
        throw new BadRequestException('bookingId is required');
      }

      const booking = await this.bookingsService.getPujaByBookingAndUserId(
        parseInt(bookingId, 10)
      );

      if (!booking) {
        throw new APIFailedException('Booking not found');
      }

      return {
        status: 'success',
        data: booking,
      };
    } catch (error) {
      this.logger.error('Failed to get puja by ID:', error);
      throw new APIFailedException(error);
    }
  }

  @Post('puja-booking')
  @ApiOperation({ summary: 'Create Puja booking' })
  @ApiCreatedResponse({ type: CreateBookingsDto })
  async createBookings(
    @Body() body: CreateBookingPayload
  ): Promise<BookingsArrayResponse> {
    try {
      const result = await this.bookingsService.postBookings(body);

      if (!result) {
        throw new APIFailedException('Failed to create booking');
      }

      return {
        status: 'success',
        data: result,
      };
    } catch (error) {
      this.logger.error('Failed to create booking:', error);
      throw new APIFailedException(error);
    }
  }

  @Put('puja-booking/:bookingId')
  @ApiOperation({ summary: 'Edit Puja booking by pujari' })
  @ApiCreatedResponse({ type: responseMessage })
  async editBookings(
    @Param('bookingId', new ParseIntPipe()) id: number,
    @Body() body: EditBookingPayload
  ): Promise<{ status: string; message: string }> {
    const result = await this.bookingsService.editBooking(id, body);

    if (!result) {
      throw new APIFailedException('Failed to edit booking');
    }

    return sendResponseMessage(result);
  }

  @Post('update-status')
  @ApiOperation({ summary: 'API for updating the status of the booking' })
  @ApiBody({ type: UpdateBookingPayload })
  @ApiCreatedResponse({ type: UpdateBookingResponse })
  async updatePujaStatus(
    @Body() body: UpdateBookingPayload
  ): Promise<UpdatePujaStatusResponse> {
    try {
      this.logger.debug('API Triggered - Update puja status');
      const result = await this.bookingsService.updatePujaStatus(body);

      if (!result) {
        throw new APIFailedException('Failed to update puja status');
      }

      return {
        status: 'success',
        data: result,
      };
    } catch (error) {
      this.logger.error('Failed to update puja status:', error);
      throw new APIFailedException(error);
    }
  }

  @Put('update-status/:bookingId')
  @ApiOperation({
    summary: 'API for updating the different fields status of the booking',
  })
  @ApiBody({ type: UpdateBookingStatuses })
  @ApiCreatedResponse({ type: responseMessage })
  async updateBookingStatuses(
    @Param('bookingId', new ParseIntPipe()) bookingId: number,
    @Body() body: UpdateBookingStatuses
  ): Promise<BookingStatusResponse> {
    try {
      this.logger.debug('API Triggered - Update booking status');
      const result = await this.bookingsService.updateBookingStatuses(
        bookingId,
        body
      );

      if (!result) {
        throw new APIFailedException('Failed to update booking status');
      }

      // Handle both string and object responses
      const message =
        typeof result === 'string'
          ? result
          : result.message || 'Status updated successfully';

      return {
        status: 'success',
        message: message,
      };
    } catch (error) {
      this.logger.error('Failed to update booking status:', error);
      throw new APIFailedException(error);
    }
  }

  @Get('booking-details')
  @ApiOperation({
    summary: 'API for getting booking details based on the booking ID',
  })
  @ApiOkResponse({ type: Bookings })
  async getBookingDetails(
    @Query() query: BookingDetailsQuery
  ): Promise<BookingDetailsResponse> {
    try {
      this.logger.debug(
        'API Triggered - Getting booking details based on the booking ID'
      );

      const result = await this.bookingsService.getBookingDetails(query);

      if (!result) {
        throw new APIFailedException('Booking not found');
      }

      // Check if result is an array and handle accordingly
      if (Array.isArray(result)) {
        if (result.length === 0) {
          throw new APIFailedException('Booking not found');
        }
        return {
          status: 'success',
          data: result[0], // Take first booking if array
        };
      }

      // Check if result is an error
      if (result instanceof APIFailedException) {
        throw result;
      }

      // At this point TypeScript knows result must be a Booking
      return {
        status: 'success',
        data: result,
      };
    } catch (error) {
      this.logger.error('Failed to get booking details:', error);
      throw new APIFailedException(error);
    }
  }
}
