import { Injectable, Logger } from '@nestjs/common';
import { KaruserPujariMappings } from 'src/entities/karuser_pujari_mappings';
import {
  Categories,
  Cities,
  Countries,
  Languages,
  Nakshatram,
  Pujas,
  PujaSamagriMappings,
  States,
} from 'src/entities';
import { PujaModels } from 'src/entities/puja-models';
import { AppDataSource } from './../common/db-utils';
import { APIFailedException } from './../common/exceptions';
import { GetPujaModelsDto, GetPujasPayload } from './models/pujas.dto';

@Injectable()
export class PujariService {
  logger: Logger;

  constructor() {
    this.logger = new Logger();
  }

  async getPujaModels(query: GetPujaModelsDto): Promise<PujaModels[] | null> {
    let filter: any = query.pujaId ? { pujaId: query.pujaId } : {};
    filter = query.id ? { id: query.id, ...filter } : { ...filter };

    const relations = query.id ? ['procedures', 'procedures.procedure'] : [];

    try {
      this.logger.debug('API Triggered');
      const repo = AppDataSource.getRepository(PujaModels);
      return await repo.find({
        where: { ...filter },
        relations: [...relations],
      });

      //return this.appService.getHello();}
    } catch (error: any) {
      this.logger.error(error);
      throw new APIFailedException(error);
    }
  }

  async getPujas(body: GetPujasPayload): Promise<Pujas[] | null> {
    try {
      const repo = AppDataSource.getRepository(Pujas);

      let query = repo
        .createQueryBuilder('pujas')
        .leftJoinAndSelect(
          'pujas.pujacategorymappings',
          'mapping',
          'mapping.puja_id = pujas.id'
        )
        .innerJoinAndSelect(
          'pujas.pujamodels',
          'pujamodels',
          'pujamodels.puja_id = pujas.id'
        )
        .leftJoinAndSelect(
          'pujas.pujaSamagriMappings',
          'pujaSamagriMappings',
          'pujaSamagriMappings.pujaId = pujas.id'
        )
        .leftJoinAndSelect(
          'pujamodels.procedures',
          'procedures',
          'procedures.modelId = pujamodels.id'
        )
        .leftJoinAndSelect(
          'procedures.procedure',
          'procedure',
          'procedure.id = procedures.procedureId'
        )
        .leftJoinAndSelect(
          'pujaSamagriMappings.samagri',
          'samagri',
          'pujaSamagriMappings.samagriId = samagri.id'
        )
        .orderBy('procedures.order', 'ASC')

        .where('pujamodels.is_popular_model = 1');

      if (body.history !== undefined && body.history) {
        query = query.leftJoinAndSelect(
          'pujas.pujaHistory',
          'history',
          'history.puja_id = pujas.id'
        );
      }

      if (body.id !== undefined) {
        query = query.andWhere(`pujas.id = '${body.id}'`);

        return await query.getMany();
      }

      if (body.mode) {
        query = query.andWhere(
          `pujas.mode1 = '${body.mode}' or pujas.mode2 = '${body.mode}' or pujas.mode3 = '${body.mode}' or pujas.mode4 = '${body.mode}'`
        );
      }

      // if (body.categoryId) {
      //   query = query.andWhere(`mapping.categoryId = '${body.categoryId}'`);
      // }

      if (body.subCategoryId) {
        query = query.andWhere(
          `mapping.subCategoryId in (${body.subCategoryId}, null)`
        );
      }
      return await query.getMany();
    } catch (error: any) {
      this.logger.error(error);
      throw new APIFailedException(error);
    }
  }

  async getPujaSamagri(pujaId: number): Promise<any> {
    this.logger.log(`*** PUJA ID  ${pujaId} ******`);
    try {
      this.logger.debug('API Triggered');
      const repo = AppDataSource.getRepository(PujaSamagriMappings);
      const query = await repo
        .createQueryBuilder('pujaSamagri')
        .leftJoinAndSelect(
          'pujaSamagri.samagri',
          'samagri',
          'samagri.id = pujaSamagri.samagriId'
        )
        .where('pujaSamagri.pujaId = :pujaId', { pujaId })
        .getMany();
      this.logger.log('===>query', query);
      return query.map((item) => {
        const samagri = item.samagri;
        return {
          id: item.id,
          name: samagri.name || '',
          description: samagri.description || '',
          units: samagri.units || '',
          standardQty: samagri.standardQty || '',
          qty: item.noOfStandardQty,
          samagriType: item.samagriType,
          isProvidedBy: item.isProvidedBy,
          karishyeProvided: samagri.karishyeProvided || '',
          costPrice: samagri.costPrice || 0,
          pujariSellingPrice: samagri.pujariSellingPrice || 0,
          customerMrp: samagri.customerMrp || 0,
          samagriId: item.samagriId,
          orderId: samagri.orderId ?? 0,
        };
      });
    } catch (error: any) {
      this.logger.error(error);
      throw new APIFailedException(error);
    }
  }

  async getPujaHistory(): Promise<Pujas[] | null> {
    const relations = ['pujaHistory'];

    try {
      this.logger.debug('getPujaHistory API Triggered');
      const repo = AppDataSource.getRepository(Pujas);
      return await repo.find({
        relations: [...relations],
      });
    } catch (error: any) {
      this.logger.error(error);
      throw new APIFailedException(error);
    }
  }

  async getExistingCustomers(pujariId: number) {
    try {
      this.logger.debug('API Triggered');
      const repo = AppDataSource.getRepository(KaruserPujariMappings);
      const query = await repo
        .createQueryBuilder('KaruserPujariMappings')
        .leftJoinAndSelect(
          'KaruserPujariMappings.karuser',
          'karuser',
          'karuser.id = KaruserPujariMappings.karuser_id'
        )
        .where({ pujariId: pujariId })
        .getMany();
      return query
        .filter((user) => user.karuser !== null)
        .map((userData) => (userData.karuser ? { ...userData.karuser } : null));
    } catch (error: any) {
      this.logger.error(error);
      throw new APIFailedException(error);
    }
  }

  async getAllPujas(): Promise<Categories[] | null> {
    try {
      const categories = AppDataSource.getRepository(Categories);

      const query = categories
        .createQueryBuilder('categories')
        .leftJoinAndSelect(
          'categories.subCategories',
          'subCategories',
          'subCategories.categoryId = categories.id'
        )
        .leftJoinAndSelect(
          'subCategories.pujasubcategorymappings',
          'pujasubcategorymappings',
          'pujasubcategorymappings.subCategoryId = subCategories.id'
        )
        .leftJoinAndSelect(
          'pujasubcategorymappings.pujas',
          'pujas',
          'pujas.id = pujasubcategorymappings.pujaId'
        )
        .innerJoinAndSelect(
          'pujas.pujamodels',
          'pujamodels',
          'pujas.id = pujamodels.pujaId'
        )
        .leftJoinAndSelect(
          'pujamodels.procedures',
          'procedures',
          'procedures.modelId = pujamodels.id'
        )
        .leftJoinAndSelect(
          'procedures.procedure',
          'procedure',
          'procedure.id = procedures.procedureId'
        )
        .where('pujamodels.is_popular_model = 1');

      return await query.getMany();
    } catch (error: any) {
      this.logger.error(error);
      throw new APIFailedException(error);
    }
  }

  // async getBookings(): Promise <Bookings[] | null> {
  //   try {
  //     this.logger.debug("API Triggered");
  //     const repo = await AppDataSource.getRepository(Bookings).find()
  //     this.logger.log('repo',repo)
  //     return  repo

  //     //return this.appService.getHello();}
  //   } catch (error: any) {
  //     this.logger.error(error);
  //     throw new APIFailedException(error);
  //   }
  // }

  async getMetadata() {
    try {
      this.logger.debug('Getting website metadata');
      const countriesRepo = AppDataSource.getRepository(Countries);
      const statesRepo = AppDataSource.getRepository(States);
      const citiesRepo = AppDataSource.getRepository(Cities);
      const languagesRepo = AppDataSource.getRepository(Languages);
      const nakshatramRepo = AppDataSource.getRepository(Nakshatram);
      const [countries, states, cities, languages, nakshatram] =
        await Promise.all([
          countriesRepo.find(),
          statesRepo.find(),
          citiesRepo.find(),
          languagesRepo.find(),
          nakshatramRepo.find(),
        ]);

      return { countries, states, cities, languages, nakshatram };
    } catch (error: any) {
      this.logger.error(error);
      throw new APIFailedException(error);
    }
  }
}
