import { Injectable, Logger } from '@nestjs/common';
import { errorResponse } from 'src/common/common-utils';
import { AppDataSource } from 'src/common/db-utils';
import { APIFailedException } from 'src/common/exceptions';
import { KarishyeUsers } from 'src/entities';
import { EmailService } from 'src/pujari/email-config/email.service';
import { WhatsappService } from 'src/pujari/whatsapp/whatsapp.service';
import { UserDetailsQuery } from '../../models/createUsers.dto';
import { ContactPayload } from '../../models/twilio.dto';
import { TwilioService } from '../twilio/twilio.service';
import PdfPrinter from 'pdfmake';
import { v4 as uuid } from 'uuid';
import { BookingsService } from 'src/pujari/bookings/bookings.service';
import PDFDocument from 'pdfkit';
import { uploadFileToS3 } from '../../../../common/s3Upload';
import { Users } from 'src/entities/my-dharma-users.entity'; //import the Email entity
@Injectable()
export class UsersService {
  logger: Logger;

  constructor(
    private twilioService: TwilioService,
    private emailService: EmailService,
    private whatsappService: WhatsappService,
    private bookingService: BookingsService
  ) {
    this.logger = new Logger();
  }

  async createUser(data: any) {
    const body = data.data;
    const pujariId = data.id;
    const shouldNotify = data.isNotify || false;
    const birthDate =
      body.dateOfBirth == null ? null : new Date(body.dateOfBirth);

    // const body = data;
    // const pujariId = data.pujariId ? data.pujariId : null;
    // const shouldNotify = data.isNotify || false;
    // const birthDate = body.dateOfBirth ? body.dateOfBirth : null;

    try {
      const usersRepo = AppDataSource.getRepository(KarishyeUsers);
      let result, query;

      if (body.id) {
        query = await usersRepo
          .createQueryBuilder('users')
          .where(`users.id = '${body.id}'`)
          .getOne();

        result = await usersRepo
          .createQueryBuilder()
          .update()
          .set({
            ...body,
            id: body.id,
          })
          .where(`id = '${body.id}'`)
          .execute();
      } else {
        const userId = uuid();
        body.id = userId;
        const finalObj = pujariId
          ? {
            id: uuid(),
            pujariId: pujariId,
            karuserId: userId,
            createdAt: new Date(),
          }
          : {
            id: uuid(),
            karuserId: userId,
            createdAt: new Date(),
          };
        result = await usersRepo
          .createQueryBuilder()
          .insert()
          .into(KarishyeUsers)
          .values({
            ...body,
            dateOfBirth: birthDate,
            dateOfRegistration: new Date(),
          })
          .execute();

        this.logger.log('result', result);
        // this.whatsappService.sendWhatsappMessage(
        //   body.phoneNumber,
        //   "user_created",
        //   []
        // );

        console.log('shouldNotifyshouldNotifyshouldNotify', shouldNotify);

        if (shouldNotify) {
          console.log('==== inside if condition ====', body.phoneNumber);
          this.whatsappService.sendWhatsappMessage(
            body.phoneNumber,
            'post_enquiry_intro_msg',
            []
          );
        }

        // if (body.emailId) {
        //   this.emailService.sendMail({
        //     template: NEW_USER.testTemplate,
        //     mailProps: {
        //       to: ["phanindra@divami.com", body.emailId],
        //     },
        //     dynamicData: {},
        //   });
        // }
      }

      return { userId: body.id, ...body };
    } catch (error) {
      this.logger.error(error);
      throw new APIFailedException(error);
    }
  }

  async getUserDetails(params: UserDetailsQuery) {
    try {
      const usersRepo = AppDataSource.getRepository(KarishyeUsers);

      const data = await usersRepo.find({ where: { id: params.id } });

      if (data.length === 0) {
        return 'No user found';
      }

      return { ...data };
    } catch (error) {
      this.logger.error(error);
      throw new APIFailedException(error);
    }
  }

  uploadFileAndGetUrl = (pdfDoc: any, fileName: string) => {
    const chunks: any = [];
    return new Promise((resolve) => {
      pdfDoc.on('data', (chunk: any) => {
        chunks.push(chunk);
      });
      let url: any = '';
      let result: any = '';
      pdfDoc.on('end', () => {
        result = Buffer.concat(chunks);
        url = uploadFileToS3(result, fileName);
        resolve(url);
      });
      pdfDoc.end();
    });
  };

  contactKarishye(data: ContactPayload) {
    try {
      return this.twilioService
        .verifyOTP({
          phoneNumber: data.phoneNumber,
          otpCode: data.otpCode,
        })
        .then((res: { statusCode: number; message: string }) => {
          // if (res.statusCode === 200) {
          this.logger.log('res--', res);

          this.whatsappService.sendWhatsappMessage(
            process.env.ADMIN_NUMBER
              ? process.env.ADMIN_NUMBER
              : '918121007481',
            'contact_us_admin',
            [
              {
                name: 'customer_name',
                value: data.firstName + ' ' + data.lastName,
              },
              {
                name: 'email',
                value: data.email ? data.email : 'No email provided',
              },
              { name: 'phone_number', value: data.phoneNumber },
              {
                name: 'message',
                value: data.message ? data.message : 'No message provided',
              },
            ]
          );

          const result = this.whatsappService.sendWhatsappMessage(
            `${data.phoneNumber.replace('+', '')}`,
            'contact_us_customer',
            []
          );
          this.logger.log('result', result);

          return {
            statusCode: 200,
            message:
              'Thank you. Your puja booking is recorded with us. Our support team will get back to you within 1-2 hours and explain the next steps!',
          };
          // } else {
          //   this.logger.log(res);

          //   return res;
          // }
        })
        .catch((err: { statusCode: number; message: string }) => {
          return errorResponse(err);
        });
    } catch (error) {
      return errorResponse(new APIFailedException(error));
    }
  }

  convertStringToFormattedDate(dateString?: string) {
    if (dateString == null) {
      return '';
    }
    return new Date(dateString).toLocaleDateString('en-GB', {
      day: '2-digit',
      month: '2-digit',
      year: 'numeric',
    });
  }

  checkWhetherSamagriProvidedByUser(samagri: any) {
    const isProvidedByUser = samagri.some(
      (item: any) => item.isProvidedBy == 'user'
    );
    return isProvidedByUser;
  }

  checkWhetherSamagriProvidedByKarishye(samagri: any) {
    const isProvidedByPujariOrPujari = samagri.some(
      (item: any) =>
        item.isProvidedBy == 'karishye' || item.isProvidedBy == null
    );
    return isProvidedByPujariOrPujari;
  }

  checkWhetherSamagriProvidedByPujari(samagri: any) {
    const isProvidedByPujariOrPujari = samagri.some(
      (item: any) => item.isProvidedBy == 'pujari'
    );
    return isProvidedByPujariOrPujari;
  }

  convertDateTimeIntoTime(dateTimeString: any) {
    if (dateTimeString != null) {
      return new Date(dateTimeString).toLocaleString('en-US', {
        hour: 'numeric',
        minute: 'numeric',
        hour12: true,
      });
    } else {
      return '';
    }
  }

  getPujariName(name: any, surname: any) {
    this.logger.log('namename', name);
    this.logger.log('surnamesurname', surname);

    if (name && surname) {
      return `${name} ${surname}`;
    } else {
      return ''; // Return an empty string when either name or surname is missing
    }
  }

  getPhoneNumber(mobileNumber: any) {
    if (mobileNumber) {
      return mobileNumber;
    } else {
      return '';
    }
  }

  checkIfPujariExists(pujariData: any) {
    if (pujariData) {
      return true;
    }
    return false;
  }

  async generatePDF(data: UserDetailsQuery) {
    try {
      const [bookingDetails]: any = await this.bookingService.getBookingDetails(
        data
      );

      if (bookingDetails) {
        const fontSize10 = 10;
        const fontSize5 = 5;

        const tableMainHeaderFont = 8;
        const tablemainHeaderTextColor = '#C7B062';
        const borderColor = ['#C7B062', '#C7B062', '#C7B062', '#C7B062'];
        const cellheight = [0, 4, 0, 4];
        const centerAlign = 'center';
        const margin_2 = [0, 2, 0, 0];
        const user: any = {};
        user.userName = bookingDetails?.user?.name || '';
        user.surName = bookingDetails?.user?.surname || '';
        user.gothram = bookingDetails?.user?.gothram || '';
        user.nakshatram = bookingDetails?.user?.nakshatram || '';
        user.dateOfBirth = bookingDetails?.user?.dateOfBirth
          ? bookingDetails?.user?.dateOfBirth?.toISOString()
          : '';
        user.contact = bookingDetails?.user?.phoneNumber || '';
        user.pujaName = bookingDetails?.puja_model?.puja?.name || '';
        user.pujariKarishyeNotes = bookingDetails?.pujariKarishyeNotes || '';
        user.pujariCustomerNotes = bookingDetails?.pujariCustomerNotes || '';
        user.customerNotes = bookingDetails?.customerNotes || '';
        user.bookingAddress = bookingDetails?.address || '';
        user.bookingStatus = bookingDetails?.bookingStatus || '';
        user.poojaStartDay = bookingDetails?.startDate
          ? bookingDetails?.startDate.toISOString()
          : '';
        user.poojaEndDay = bookingDetails?.endDate
          ? bookingDetails?.endDate?.toISOString()
          : '';
        user.pujaDurationHours = bookingDetails?.durationHrs || '';
        user.muhurtam = bookingDetails?.muhurtam
          ? bookingDetails?.muhurtam?.toISOString()
          : '';
        user.noOfPujaris = bookingDetails?.noOfPujaris || '';
        user.pujariCost = bookingDetails?.pujariCost || '';
        user.bookingSellingPrice = bookingDetails?.bookingSellingPrice || '';
        // const address = `${bookingDetails?.user?.name}\n\n${bookingDetails?.user?.surname} \n\n ${bookingDetails?.user?.addressLine1} \n\n ${bookingDetails?.user?.addressLine2} \n\n ${bookingDetails?.user?.city} \n\n ${bookingDetails?.user?.state} \n\n ${bookingDetails?.user?.pincode} \n\n ${bookingDetails?.user?.contry} \n\n ${bookingDetails?.user?.phoneNumber} \n\n ${bookingDetails?.user?.emailId}`;
        user.addressLine1 = bookingDetails?.user?.addressLine1 || '';
        user.addressLine2 = bookingDetails?.user?.addressLine2 || '';
        user.city = bookingDetails?.user?.city || '';
        user.state = bookingDetails?.user?.state || '';
        user.pincode = bookingDetails?.user?.pincode || '';
        user.country = bookingDetails?.user?.contry || '';
        user.phoneNumber = bookingDetails?.user?.phoneNumber || '';
        user.emailId = bookingDetails?.user?.emailId || '';
        user.gst = bookingDetails?.gst || 0;
        user.startTime = this.convertDateTimeIntoTime(
          bookingDetails?.startDate
        );
        user.endTime = this.convertDateTimeIntoTime(bookingDetails?.endDate);
        user.muhurtamTime = this.convertDateTimeIntoTime(
          bookingDetails?.muhurtam
        );

        let samagriCost = 0;

        // Create samagri rows for samagri table for provided by samagri
        const samagriBody: any = [
          [
            {
              text: 'Item Description',
              fontSize10,
              fillColor: '#C7B062',
              borderColor: borderColor,
              margin: cellheight,
              alignment: centerAlign,
            },
            {
              text: 'Qty',
              fontSize10,
              fillColor: '#C7B062',
              borderColor: borderColor,
              margin: cellheight,
              alignment: centerAlign,
            },
            {
              text: 'Amount',
              fontSize10,
              fillColor: '#C7B062',
              borderColor: borderColor,
              margin: cellheight,
              alignment: centerAlign,
            },
          ],
        ];

        // create samagri table for pujari samagri

        const pujariSamagriBody: any = [
          [
            {
              text: 'Item Description',
              fontSize10,
              fillColor: '#C7B062',
              borderColor: borderColor,
              margin: cellheight,
              alignment: centerAlign,
            },
            {
              text: 'Qty',
              fontSize10,
              fillColor: '#C7B062',
              borderColor: borderColor,
              margin: cellheight,
              alignment: centerAlign,
            },
          ],
        ];

        // create samagri table for user samagri
        const userSamagriBody: any = [
          [
            {
              text: 'Item Description',
              fontSize10,
              fillColor: '#C7B062',
              borderColor: borderColor,
              margin: cellheight,
              alignment: centerAlign,
            },
            {
              text: 'Qty',
              fontSize10,
              fillColor: '#C7B062',
              borderColor: borderColor,
              margin: cellheight,
              alignment: centerAlign,
            },
          ],
        ];

        const samagriDetails = bookingDetails?.bookingSamagriMappings;
        let totalSamagriAmount = 0;

        samagriDetails.sort(
          (a: any, b: any) => a.samagri.orderId - b.samagri.orderId
        );

        for (let i = 0; i < samagriDetails.length; i++) {
          const samagriRow = [];
          const pujariSamagriRow = [];
          const userSamagriRow = [];

          let amount = 0;
          const providedBy = samagriDetails[i].isProvidedBy;
          const samagriTitle = samagriDetails[i].samagri?.name.toString();

          // we will add content of the samagri of the table here
          if (providedBy == 'karishye' || providedBy == null) {
            amount =
              samagriDetails[i].samagri?.customerMrp * samagriDetails[i].qty;

            const quantity =
              parseInt(samagriDetails[i].qty) *
              parseInt(samagriDetails[i].samagri.standardQty);
            const description =
              samagriDetails[i].samagri.description != null
                ? ` \n ${samagriDetails[i].samagri.description}`
                : '';
            samagriRow.push({
              text: `${samagriTitle} ${description}`,
              // fontSize10,
              fontSize5,
              borderColor: borderColor,
              margin: cellheight,
            });
            samagriRow.push({
              text: `${quantity} ${samagriDetails[i].samagri.units}`,
              alignment: 'right',
              fontSize10,
              borderColor: borderColor,
              margin: cellheight,
            });
            // get all over total amount
            totalSamagriAmount += amount;

            samagriCost += amount;

            samagriRow.push({
              text: amount,
              fontSize10,
              alignment: 'right',
              borderColor: borderColor,
              margin: cellheight,
            });

            // push above array into samagri body
            samagriBody.push(samagriRow);
          }

          // samagri provided by pujari
          if (providedBy == 'pujari') {
            amount =
              samagriDetails[i].samagri?.customerMrp * samagriDetails[i].qty;
            const quantity =
              parseInt(samagriDetails[i].qty) *
              parseInt(samagriDetails[i].samagri.standardQty);
            const description =
              samagriDetails[i].samagri.description != null
                ? ` \n ${samagriDetails[i].samagri.description}`
                : '';

            pujariSamagriRow.push({
              text: `${samagriTitle} ${description}`,
              // fontSize10,
              fontSize5,
              borderColor: borderColor,
              margin: cellheight,
            });

            pujariSamagriRow.push({
              text: `${quantity} ${samagriDetails[i].samagri.units}`,
              alignment: 'right',
              fontSize10,
              borderColor: borderColor,
              margin: cellheight,
            });

            pujariSamagriBody.push(pujariSamagriRow);
          }
          // samagri provided by user
          if (providedBy == 'user') {
            amount =
              samagriDetails[i].samagri?.customerMrp * samagriDetails[i].qty;
            const quantity =
              parseInt(samagriDetails[i].qty) *
              parseInt(samagriDetails[i].samagri.standardQty);
            const description =
              samagriDetails[i].samagri.description != null
                ? ` \n ${samagriDetails[i].samagri.description}`
                : '';

            userSamagriRow.push({
              text: `${samagriTitle} ${description}`,
              // fontSize10,
              fontSize5,
              borderColor: borderColor,
              margin: cellheight,
            });
            userSamagriRow.push({
              text: `${quantity} ${samagriDetails[i].samagri.units}`,
              alignment: 'right',
              fontSize10,
              borderColor: borderColor,
              margin: cellheight,
            });
            userSamagriBody.push(userSamagriRow);
          }
        }
        // create procedure list

        // procedure tabel titles
        const procedureBody: any = [
          [
            {
              text: 'sl.no',
              fontSize10,
              fillColor: '#C7B062',
              borderColor: borderColor,
              margin: cellheight,
              alignment: centerAlign,
            },
            {
              text: 'Name',
              fontSize10,
              fillColor: '#C7B062',
              borderColor: borderColor,
              margin: cellheight,
              alignment: centerAlign,
            },
          ],
        ];

        const procedure: any = bookingDetails?.bookingproceduremappings;
        procedure.sort((item1: any, item2: any) => item1.order - item2.order);
        for (let i = 0; i < procedure.length; i++) {
          const procedureRow = [];
          procedureRow.push({
            text: `${i + 1} `,
            // fontSize10,
            fontSize5,
            borderColor: borderColor,
            margin: cellheight,
            fonts: 'BalooTammudu',
          });
          const procedureTitle = procedure[i]?.procedure?.title || '';
          procedureRow.push({
            text: procedureTitle,
            alignment: 'left',
            fontSize10,
            borderColor: borderColor,
            margin: cellheight,
          });
          procedureBody.push(procedureRow);
        }

        const procedureList: any = [];
        if (procedure) {
          for (
            let procedureCount = 0;
            procedureCount <= procedure.length;
            procedureCount++
          ) {
            procedureList.push({
              text: procedure[procedureCount]?.procedure?.title,
              fontSize10,
              margin: [10, 10, 10, 10],
              color: 'grey',
            });
          }
        }

        // create total row for samagri table
        const totalRow = [
          {
            text: 'Total',
            alignment: 'right',
            colSpan: 2,
            fontSize10,
            borderColor: borderColor,
            margin: cellheight,
          },
          {},
          {
            text: totalSamagriAmount,
            alignment: 'right',
            fontSize10,
            borderColor: borderColor,
            margin: cellheight,
          },
        ];
        samagriBody.push(totalRow);

        const fonts = {
          Helvetica: {
            normal: 'Helvetica',
            bold: 'Helvetica-Bold',
            italics: 'Helvetica-Oblique',
            bolditalics: 'Helvetica-BoldOblique',
          },
          BalooTammudu: {
            normal: 'assets/fonts/BalooTammudu-Medium.ttf',
            bold: 'assets/fonts/BalooTammudu-Medium.ttf',
            italics: 'assets/fonts/BalooTammudu-Medium.ttf',
            bolditalics: 'assets/fonts/BalooTammudu-Medium.ttf',
          },
          // NotoSansTelugu: {
          //   normal: "assets/fonts/NotoSansTelugu-Regular.ttf",
          //   bold: "assets/fonts/NotoSansTelugu-Regular.ttf",
          //   italics: "assets/fonts/NotoSansTelugu-Regular.ttf",
          //   bolditalics: "assets/fonts/NotoSansTelugu-Regular.ttf",
          // },
        };
        const printer = new PdfPrinter(fonts);

        const docDefinition: any = {
          content: [
            // logo
            {
              image: 'logo.png',
              fit: [90, 90],
              alignment: 'center',
            },

            //  title
            {
              text: 'B O O K I N G  D E T A I L S \n ',
              fontSize: 15,
              margin: [0, 20],
            },

            {
              text: `Booking Id: ${data.id}`,
              alignment: 'right',
              margin: [0, 20],
            },
            //  content
            { text: 'To: \n ', fontSize: fontSize10 },
            {
              text: `${user.userName} ${user.surName}`,
              fontSize10,
              margin: margin_2,
            },
            { text: user.addressLine1, fontSize10, margin: margin_2 },
            { text: user.addressLine2, fontSize10, margin: margin_2 },
            { text: user.city, fontSize10, margin: margin_2 },
            { text: user.state, fontSize10, margin: margin_2 },
            { text: user.pincode, fontSize10, margin: margin_2 },
            { text: user.country, fontSize10, margin: margin_2 },
            { text: user.phoneNumber, fontSize10, margin: margin_2 },
            { text: user.emailId, fontSize10, margin: [0, 0, 0, 20] },

            // customer details table heading
            {
              style: 'tableExample',
              table: {
                widths: ['*'],
                heights: [20],
                body: [
                  [
                    {
                      text: 'C U S T O M E R  D E T A I L S',
                      fontSize: tableMainHeaderFont,
                      fillColor: '#50223C',
                      border: [false, false, false, false],
                      margin: [0, 8, 0, 0],
                      color: tablemainHeaderTextColor,
                    },
                  ],
                ],
              },
              margin: [0, 2],
            },
            // customer details table content
            {
              style: 'tableExample',
              margin: [0, 0, 0, 20],
              table: {
                widths: '*',
                body: [
                  [
                    {
                      text: 'Label',
                      fontSize10,
                      fillColor: '#C7B062',
                      borderColor: borderColor,
                      margin: cellheight,
                    },
                    {
                      text: 'Detail',
                      fontSize10,
                      fillColor: '#C7B062',
                      borderColor: ['#C7B062', '#C7B062', '#C7B062', '#C7B062'],
                      margin: cellheight,
                    },
                  ],
                  [
                    {
                      text: 'Name',
                      fontSize10,
                      borderColor: ['#C7B062', '#C7B062', '#C7B062', '#C7B062'],
                      margin: cellheight,
                    },
                    {
                      text: user.userName,
                      fontSize10,
                      borderColor: ['#C7B062', '#C7B062', '#C7B062', '#C7B062'],
                      margin: cellheight,
                    },
                  ],
                  [
                    {
                      text: 'Surname',
                      fontSize10,
                      borderColor: ['#C7B062', '#C7B062', '#C7B062', '#C7B062'],
                      margin: cellheight,
                    },
                    {
                      text: user.surName,
                      fontSize10,
                      borderColor: ['#C7B062', '#C7B062', '#C7B062', '#C7B062'],
                      margin: cellheight,
                    },
                  ],
                  [
                    {
                      text: 'Gothram',
                      fontSize10,
                      borderColor: ['#C7B062', '#C7B062', '#C7B062', '#C7B062'],
                      margin: cellheight,
                    },
                    {
                      text: user.gothram,
                      fontSize10,
                      borderColor: ['#C7B062', '#C7B062', '#C7B062', '#C7B062'],
                      margin: cellheight,
                    },
                  ],
                  [
                    {
                      text: 'Nakshatram',
                      fontSize10,
                      borderColor: ['#C7B062', '#C7B062', '#C7B062', '#C7B062'],
                      margin: cellheight,
                    },
                    {
                      text: user.nakshatram,
                      fontSize10,
                      borderColor: ['#C7B062', '#C7B062', '#C7B062', '#C7B062'],
                      margin: cellheight,
                    },
                  ],
                  [
                    {
                      text: 'Date of birth',
                      fontSize10,
                      borderColor: ['#C7B062', '#C7B062', '#C7B062', '#C7B062'],
                      margin: cellheight,
                    },
                    {
                      text: user.dateOfBirth
                        ? this.convertStringToFormattedDate(user.dateOfBirth)
                        : '',
                      borderColor: ['#C7B062', '#C7B062', '#C7B062', '#C7B062'],
                      margin: cellheight,
                    },
                  ],
                  [
                    {
                      text: 'Mobile',
                      fontSize10,
                      borderColor: ['#C7B062', '#C7B062', '#C7B062', '#C7B062'],
                      margin: cellheight,
                    },
                    {
                      text: user.contact,
                      fontSize10,
                      borderColor: ['#C7B062', '#C7B062', '#C7B062', '#C7B062'],
                      margin: cellheight,
                    },
                  ],
                ],
              },
            },

            // booking details heading
            {
              style: 'tableExample',
              table: {
                widths: ['*'],
                heights: [20],
                body: [
                  [
                    {
                      text: 'B O O K I N G  D E T A I L S',
                      fontSize: tableMainHeaderFont,
                      fillColor: '#50223C',
                      border: [false, false, false],
                      margin: [0, 8, 0, 0],
                      color: tablemainHeaderTextColor,
                    },
                  ],
                ],
              },
              margin: [0, 2],
            },

            // booking details table
            {
              style: 'tableExample',
              margin: [0, 0, 0, 20],
              table: {
                widths: '*',
                body: [
                  [
                    {
                      text: 'Label',
                      fontSize10,
                      fillColor: '#C7B062',
                      margin: cellheight,
                      borderColor: borderColor,
                    },
                    {
                      text: 'Detail',
                      fontSize10,
                      fillColor: '#C7B062',
                      margin: cellheight,
                      borderColor: borderColor,
                    },
                  ],
                  [
                    {
                      text: 'Puja Name',
                      fontSize10,
                      borderColor: borderColor,
                      margin: cellheight,
                    },
                    {
                      text: user.pujaName,
                      fontSize10,
                      borderColor: borderColor,
                      margin: cellheight,
                    },
                  ],
                  [
                    {
                      text: 'Pujari Name',
                      fontSize10,
                      borderColor: borderColor,
                      margin: cellheight,
                    },
                    {
                      text: this.checkIfPujariExists(bookingDetails.pujaris)
                        ? `${bookingDetails.pujaris.name} ${bookingDetails.pujaris.surname}`
                        : '',
                      fontSize10,
                      borderColor: borderColor,
                      margin: cellheight,
                    },
                  ],
                  [
                    {
                      text: 'Pujari Number',
                      fontSize10,
                      borderColor: borderColor,
                      margin: cellheight,
                    },
                    {
                      text: this.checkIfPujariExists(bookingDetails.pujaris)
                        ? bookingDetails.pujaris.phoneNumber
                        : '',
                      fontSize10,
                      borderColor: borderColor,
                      margin: cellheight,
                    },
                  ],

                  [
                    {
                      text: 'Booking Address',
                      fontSize10,
                      borderColor: borderColor,
                      margin: cellheight,
                    },
                    {
                      text: user.bookingAddress,
                      fontSize10,
                      borderColor: borderColor,
                      margin: cellheight,
                    },
                  ],
                  [
                    {
                      text: 'Booking Status',
                      fontSize10,
                      borderColor: borderColor,
                      margin: cellheight,
                    },
                    {
                      text: user.bookingStatus,
                      fontSize10,
                      borderColor: borderColor,
                      margin: cellheight,
                    },
                  ],
                  [
                    {
                      text: 'No Of Pujaris',
                      fontSize10,
                      borderColor: borderColor,
                      margin: cellheight,
                    },
                    {
                      text: user.noOfPujaris,
                      fontSize10,
                      borderColor: borderColor,
                      margin: cellheight,
                    },
                  ],

                  [
                    {
                      text: 'Puja Duration Hours',
                      fontSize10,
                      borderColor: borderColor,
                      margin: cellheight,
                    },
                    {
                      text: user.pujaDurationHours,
                      fontSize10,
                      borderColor: borderColor,
                      margin: cellheight,
                    },
                  ],
                  [
                    {
                      text: 'Muhurat',
                      fontSize10,
                      borderColor: borderColor,
                      margin: cellheight,
                    },
                    {
                      text: user.muhurtam
                        ? `${this.convertStringToFormattedDate(
                          user?.muhurtam
                        )}   ${user.muhurtamTime}`
                        : '',
                      fontSize10,
                      borderColor: borderColor,
                      margin: cellheight,
                    },
                  ],

                  [
                    {
                      text: 'Pooja Start Date',
                      fontSize10,
                      borderColor: borderColor,
                      margin: cellheight,
                    },
                    {
                      text: user.poojaStartDay
                        ? `${this.convertStringToFormattedDate(
                          user.poojaStartDay?.toString()
                        )}  ${user.startTime}`
                        : '',
                      fontSize10,
                      borderColor: borderColor,
                      margin: cellheight,
                    },
                  ],
                  [
                    {
                      text: 'Pooja End Date',
                      fontSize10,
                      borderColor: borderColor,
                      margin: cellheight,
                    },
                    {
                      text: user.poojaEndDay
                        ? `${this.convertStringToFormattedDate(
                          user.poojaEndDay?.toString()
                        )}  ${user.endTime}`
                        : '',
                      fontSize10,
                      borderColor: borderColor,
                      margin: cellheight,
                    },
                  ],
                ],
              },
            },
          ],

          defaultStyle: {
            font: 'BalooTammudu',
          },
        };

        if (procedure.length > 0) {
          //  procedure table heading
          const procedureTable = {
            style: 'tableExample',
            table: {
              widths: ['*'],
              heights: [20],
              body: [
                [
                  {
                    text: 'P U J A  P R O C E D U R E',
                    fontSize: tableMainHeaderFont,
                    fillColor: '#50223C',
                    border: [false, false, false],
                    margin: [0, 8, 0, 0],
                    color: tablemainHeaderTextColor,
                  },
                ],
              ],
            },
            margin: [0, 2],
          };
          //  procedure table content
          const procedureTableContent = {
            style: 'tableExample',
            margin: [0, 0, 0, 20],
            table: {
              widths: [200, 295],
              body: procedureBody,
            },
          };
          docDefinition.content.push(procedureTable);
          docDefinition.content.push(procedureTableContent);
        }

        if (user.pujariKarishyeNotes) {
          //  pujari karishye notes heading
          const pujariKarishyeNotesHeading = {
            style: 'tableExample',
            table: {
              widths: ['*'],
              heights: [20],
              body: [
                [
                  {
                    text: 'P U J A R I  K A R I S H Y E  N O T E S',
                    fontSize: tableMainHeaderFont,
                    fillColor: '#50223C',
                    border: [false, false, false],
                    margin: [0, 8, 0, 0],
                    color: tablemainHeaderTextColor,
                  },
                ],
              ],
            },
            margin: [0, 2],
          };

          // pujari karishye notes content
          const pujariKarishyeNotesContent = {
            text: user.pujariKarishyeNotes,
            fontSize10,
            margin: [15, 10, 0, 0],
          };

          docDefinition.content.push(pujariKarishyeNotesHeading);
          docDefinition.content.push(pujariKarishyeNotesContent);
        }

        if (user.pujariCustomerNotes) {
          //  pujari customer notes heading
          const pujariCustomerNotesHeading = {
            style: 'tableExample',
            table: {
              widths: ['*'],
              heights: [20],
              body: [
                [
                  {
                    text: 'P U J A R I  C U S T O M E R  N O T E S',
                    fontSize: tableMainHeaderFont,
                    fillColor: '#50223C',
                    border: [false, false, false],
                    margin: [0, 8, 0, 0],
                    color: tablemainHeaderTextColor,
                  },
                ],
              ],
            },
            margin: [0, 2],
          };

          //  pujari customer noted content
          const pujariCustomerNotesContent = {
            text: user.pujariCustomerNotes,
            fontSize10,
            margin: [15, 10, 0, 0],
          };

          docDefinition.content.push(pujariCustomerNotesHeading);
          docDefinition.content.push(pujariCustomerNotesContent);
        }

        if (user.customerNotes) {
          //  customer notes heading
          const customerNotesHeading = {
            style: 'tableExample',
            table: {
              widths: ['*'],
              heights: [20],
              body: [
                [
                  {
                    text: 'C U S T O M E R  N O T E S',
                    fontSize: tableMainHeaderFont,
                    fillColor: '#50223C',
                    border: [false, false, false],
                    margin: [0, 8, 0, 0],
                    color: tablemainHeaderTextColor,
                  },
                ],
              ],
            },
            margin: [0, 2],
          };

          // customer notes content
          const customerNotesContent = {
            text: user.customerNotes,
            fontSize10,
            margin: [15, 10, 0, 0],
          };

          docDefinition.content.push(customerNotesHeading);
          docDefinition.content.push(customerNotesContent);
        }

        if (
          samagriDetails.length > 0 &&
          this.checkWhetherSamagriProvidedByKarishye(samagriDetails)
        ) {
          // samagri table heading
          const samagriTableHeading = {
            style: 'tableExample',
            table: {
              widths: ['*'],
              heights: [20],
              body: [
                [
                  {
                    text: 'S A M A G R I   P R O V I D E D   B Y    K A R I S H Y E',
                    fontSize: tableMainHeaderFont,
                    fillColor: '#50223C',
                    border: [false, false, false],
                    margin: [0, 8, 0, 0],
                    color: tablemainHeaderTextColor,
                  },
                ],
              ],
            },
            margin: [0, 2],
          };

          //  samagri table content
          const samagriTableContent = {
            style: 'tableExample',
            margin: [0, 0, 0, 20],
            table: {
              widths: [252, 117, 117],
              body: samagriBody,
            },
          };
          docDefinition.content.push(samagriTableHeading);
          docDefinition.content.push(samagriTableContent);
        }

        if (
          samagriDetails.length > 0 &&
          this.checkWhetherSamagriProvidedByPujari(samagriDetails)
        ) {
          const pujariSamagriTableHeading = {
            style: 'tableExample',
            table: {
              widths: ['*'],
              heights: [20],
              body: [
                [
                  {
                    text: 'S A M A G R I   P R O V I D E D   B Y   P U J A R I',
                    fontSize: tableMainHeaderFont,
                    fillColor: '#50223C',
                    border: [false, false, false],
                    margin: [0, 8, 0, 0],
                    color: tablemainHeaderTextColor,
                  },
                ],
              ],
            },
            margin: [0, 2],
          };

          const pujariSamagriTableContent = {
            style: 'tableExample',
            margin: [0, 0, 0, 20],
            table: {
              widths: [345, 150],
              body: pujariSamagriBody,
            },
          };

          docDefinition.content.push(pujariSamagriTableHeading);
          docDefinition.content.push(pujariSamagriTableContent);
        }

        if (
          samagriDetails.length > 0 &&
          this.checkWhetherSamagriProvidedByUser(samagriDetails)
        ) {
          // samagri table heading
          const userSamagriTableHeading = {
            style: 'tableExample',
            table: {
              widths: ['*'],
              heights: [20],
              body: [
                [
                  {
                    text: 'S A M A G R I   T O   K E E P   R E A D Y   A T   H O M E',
                    fontSize: tableMainHeaderFont,
                    fillColor: '#50223C',
                    border: [false, false, false],
                    margin: [0, 8, 0, 0],
                    color: tablemainHeaderTextColor,
                  },
                ],
              ],
            },
            margin: [0, 2],
          };

          //  samagri table content
          const userSamagriTableContent = {
            style: 'tableExample',
            margin: [0, 0, 0, 20],
            table: {
              widths: [345, 150],
              body: userSamagriBody,
            },
          };
          docDefinition.content.push(userSamagriTableHeading);
          docDefinition.content.push(userSamagriTableContent);
        }

        //  billling details heading
        const billingTable = {
          style: 'tableExample',
          table: {
            widths: ['*'],
            heights: [20],
            body: [
              [
                {
                  text: 'B I L L I N G  D E T A I L S',
                  fontSize: tableMainHeaderFont,
                  fillColor: '#50223C',
                  border: [false, false, false],
                  margin: [0, 8, 0, 0],
                  color: tablemainHeaderTextColor,
                },
              ],
            ],
          },
          margin: [0, 2],
        };

        const billingTableContent = {
          style: 'tableExample',
          margin: [0, 0, 0, 20],
          table: {
            widths: '*',
            body: [
              [
                {
                  text: 'Label',
                  fontSize10,
                  fillColor: '#C7B062',
                  margin: cellheight,
                  borderColor: borderColor,
                },
                {
                  text: 'Detail',
                  margin: cellheight,
                  borderColor: borderColor,
                  fontSize10,
                  fillColor: '#C7B062',
                  alignment: 'left',
                },
              ],
              [
                {
                  text: 'Pujari Cost',
                  fontSize10,
                  borderColor: borderColor,
                  margin: cellheight,
                },
                {
                  text: user.pujariCost,
                  fontSize10,
                  alignment: 'left',
                  borderColor: borderColor,
                  margin: cellheight,
                },
              ],
              [
                {
                  text: 'Samagri Cost',
                  fontSize10,
                  borderColor: borderColor,
                  margin: cellheight,
                },
                {
                  text: samagriCost,
                  fontSize10,
                  alignment: 'left',
                  borderColor: borderColor,
                  margin: cellheight,
                },
              ],
              [
                {
                  text: 'Karishye Services charges',
                  fontSize10,
                  borderColor: borderColor,
                  margin: cellheight,
                },
                {
                  text:
                    user.bookingSellingPrice -
                    samagriCost -
                    user.pujariCost -
                    user.gst,
                  fontSize10,
                  alignment: 'left',
                  borderColor: borderColor,
                  margin: cellheight,
                },
              ],

              // [
              //   {
              //     text: "GST charge",
              //     fontSize10,
              //     borderColor: borderColor,
              //     margin: cellheight,
              //   },
              //   {
              //     text: user.gst,
              //     fontSize10,
              //     alignment: "left",
              //     borderColor: borderColor,
              //     margin: cellheight,
              //   },
              // ],
              [
                {
                  text: 'Total Cost',
                  fontSize10,
                  borderColor: borderColor,
                  margin: cellheight,
                },
                {
                  text: user.bookingSellingPrice,
                  fontSize10,
                  alignment: 'left',
                  borderColor: borderColor,
                  margin: cellheight,
                },
              ],
            ],
          },
        };

        docDefinition.content.push(billingTable);
        docDefinition.content.push(billingTableContent);

        const billingDetails = {
          text: 'THANK YOU FOR YOUR BOOKING \n\n You can either pay half of the puja amount or the full amount to confirm the booking.. \n\n Pay using any of the following methods: \n\n  Gpay / Phonepe / Paytm to Karishye Private Limited. \n\n Phone no : 8121007481 \n\n UPI handle : 8121007481@okbizaxis \n\n Please msg us or call us if you need any help in making the payment .. \n\n WITH REGARDS, KARISHYE TEAM \n\n',
          fontSize10,
        };

        docDefinition.content.push(billingDetails);

        const thankingImage = {
          image: 'download.png',
          fit: [45, 45],
        };

        docDefinition.content.push(thankingImage);

        // if (procedureList.length > 0) {
        //   docDefinition.content.push({
        //     text: "Puja Procedure",
        //     bold: true,
        //     fontSize10,
        //     margin: [0, 30, 0, 0],
        //   });
        //   docDefinition.content.push({ ul: procedureList });
        // }
        const options = {};
        const fileName = `karishye_${new Date().getMilliseconds()}_${
          data.id
        }.pdf`;
        // let fileName = 'karishye.pdf';

        const pdfDoc = printer.createPdfKitDocument(docDefinition, options);

        /**
         * uncoment below code to generate pdf in local system
         *  pdfDoc.pipe(fs.createWriteStream(fileName));
            pdfDoc.end();
         */
        // pdfDoc.pipe(fs.createWriteStream(fileName));
        // pdfDoc.end();

        // comment below line if generating pdf in local
        const url: any = await this.uploadFileAndGetUrl(pdfDoc, fileName);
        return { fileName: url?.Location, phoneNumber: user.contact };
      } else {
        return { message: 'data not found' };
      }
    } catch (error: any) {
      this.logger.log('error =====>', error);
      return { error: error, message: `${error}` };
    }
  }

  async generateInvoicePDF(data: any): Promise<Buffer> {
    // API to generate invoice pdf for the particular booking to download in the frontend
    return new Promise((resolve, reject) => {
      try {
        const doc = new PDFDocument({
          size: 'A4',
          margin: 50,
        });

        const chunks: Buffer[] = [];
        doc.on('data', (chunk: Buffer) => chunks.push(chunk));
        doc.on('end', () => resolve(Buffer.concat(chunks)));

        const headerColor = '#000000';
        const textColor = '#243A73';
        const tableHeaderBgColor = '#A02334';
        const tableHeaderTextColor = '#FFFFFF';
        const tableBorderColor = '#E5E1DA';

        const outerMargin = 20;
        const pageWidth = doc.page.width;
        const pageHeight = doc.page.height;
        const innerMargin = 35;
        const padding = 10;
        const titlePadding = 60;
        doc
          .rect(
            outerMargin,
            outerMargin,
            pageWidth - 2 * outerMargin,
            pageHeight - 2 * outerMargin
          )
          .strokeColor(headerColor)
          .lineWidth(1)
          .stroke();

        doc
          .rect(
            outerMargin,
            outerMargin,
            pageWidth - 2 * outerMargin,
            innerMargin - outerMargin
          )
          .fillColor('#A02334')
          .fill();

        doc
          .rect(
            outerMargin,
            pageHeight - innerMargin,
            pageWidth - 2 * outerMargin,
            innerMargin - outerMargin
          )
          .fillColor('#A02334')
          .fill();

        doc
          .rect(
            innerMargin,
            innerMargin,
            pageWidth - 2 * innerMargin,
            pageHeight - 2 * innerMargin
          )
          .strokeColor(headerColor)
          .lineWidth(1)
          .stroke();
        doc
          .rect(
            outerMargin,
            outerMargin,
            pageWidth - 2 * outerMargin,
            pageHeight - 2 * outerMargin
          )
          .strokeColor(headerColor)
          .lineWidth(1)
          .stroke();

        doc
          .rect(
            innerMargin,
            innerMargin,
            pageWidth - 2 * innerMargin,
            pageHeight - 2 * innerMargin
          )
          .strokeColor(headerColor)
          .lineWidth(1)
          .stroke();

        const titleYPosition = innerMargin + padding + titlePadding;
        const titleXPosition = innerMargin + padding + 75;
        doc
          .fillColor(headerColor)
          .fontSize(20)
          .font('Helvetica-Bold')
          .text('TAX INVOICE', titleXPosition, titleYPosition);

        const titleWidth = doc.widthOfString('TAX INVOICE');
        doc
          .moveTo(titleXPosition, titleYPosition + 25)
          .lineTo(titleXPosition + titleWidth, titleYPosition + 25)
          .stroke();

        const logoXPosition = titleXPosition + titleWidth + 180;
        const logoYPosition = titleYPosition - 10;
        const logoWidth = 80;
        const logoHeight = 35;

        doc.image('assets/images/logo.png', logoXPosition, logoYPosition, {
          width: logoWidth,
          height: logoHeight,
        });

        const headerStartY = titleYPosition + 90;
        const headerBlockY = headerStartY - 20;
        const headerBlockHeight = 100;
        const dateBlockY = headerBlockY;
        const dateBlockHeight = 100;
        doc
          .rect(
            innerMargin + padding - 5,
            headerBlockY,
            (pageWidth - 2 * (innerMargin + padding)) / 2 + 95,
            headerBlockHeight
          )
          .strokeColor(headerColor)
          .lineWidth(1)
          .stroke();

        doc
          .fillColor(headerColor)
          .fontSize(10)
          .font('Helvetica-Bold')
          .text('KARISHYE PRIVATE LIMITED', innerMargin + padding, headerStartY)
          .fillColor(textColor)
          .fontSize(9)
          .font('Helvetica-Bold')
          .text(
            'H-802, Aparna Cyberzon,',
            innerMargin + padding,
            headerStartY + 15
          )
          .text(
            'Nallagandla, Hyderabad 500019',
            innerMargin + padding,
            headerStartY + 30
          )
          .text(
            'GSTIN: 36AAJCK3293M1ZY',
            innerMargin + padding,
            headerStartY + 45
          )
          .text(
            'CIN: U72900TG2022PTC159676',
            innerMargin + padding,
            headerStartY + 60
          );

        doc
          .rect(
            pageWidth / 2 + padding + 80,
            dateBlockY,
            (pageWidth - 2 * (innerMargin + padding)) / 2 - 80,
            dateBlockHeight
          )
          .strokeColor(headerColor)
          .lineWidth(1)
          .stroke();

        const currentDate = new Date().toLocaleDateString('en-IN');
        const rightHeaderStartY = headerStartY;
        doc
          .fillColor(textColor)
          .text(
            'DATE: ' + currentDate,
            pageWidth / 2 + padding + 90,
            rightHeaderStartY
          )
          .text(
            'INVOICE NO: 7A',
            pageWidth / 2 + padding + 90,
            rightHeaderStartY + 15
          )
          .text(
            'STATE CODE:',
            pageWidth / 2 + padding + 90,
            rightHeaderStartY + 30
          )
          .text(
            'M.No: 9985806208',
            pageWidth / 2 + padding + 90,
            rightHeaderStartY + 45
          )
          .text(
            'WWW.KARISHYE.CO',
            pageWidth / 2 + padding + 90,
            rightHeaderStartY + 60
          );

        const billToStartY = dateBlockY + 150;
        const billToBlockY = billToStartY - 50;
        const billToBlockHeight = 80;
        const billToBlockWidth = pageWidth - 2 * (innerMargin + padding);

        doc
          .rect(
            innerMargin + padding - 5,
            billToBlockY,
            billToBlockWidth - 300,
            billToBlockHeight
          )
          .strokeColor(headerColor)
          .lineWidth(1)
          .fillColor('#CBD2A4')
          .stroke();

        doc
          .rect(innerMargin + padding - 5, billToStartY - 50, 205, 15)
          .fillColor('#D8D2C2')
          .fill();

        doc
          .fillColor(headerColor)
          .text('BILL TO', innerMargin + padding, billToStartY - 45)
          .fillColor(headerColor)
          .text(
            data.billTo.name || '',
            innerMargin + padding,
            billToStartY - 20
          )
          .text(
            data.billTo.address || '',
            innerMargin + padding,
            billToStartY + 10
          );

        doc
          .moveTo(innerMargin + padding - 5, billToStartY - 35)
          .lineTo(
            innerMargin + padding - 5 + billToBlockWidth - 300,
            billToStartY - 35
          )
          .stroke();

        const tableTop = billToStartY + 90;
        const tableHeaders = ['DESCRIPTION', 'HSN/SAC', 'QTY', 'RATE', 'TOTAL'];
        const columnWidths = [200, 80, 80, 80, 80];
        let currentX = innerMargin + padding;

        const tableHeight = 190;
        doc
          .rect(
            innerMargin + padding - 5,
            tableTop - 10,
            pageWidth - 2 * (innerMargin + padding) + 10,
            tableHeight
          )
          .strokeColor(headerColor)
          .lineWidth(1)
          .stroke();

        doc
          .rect(
            currentX,
            tableTop - 5,
            pageWidth - 2 * (innerMargin + padding),
            20
          )
          .fill(tableHeaderBgColor);

        tableHeaders.forEach((header, i) => {
          doc
            .fillColor(tableHeaderTextColor)
            .fontSize(10)
            .text(header, currentX, tableTop);
          currentX += columnWidths[i];
        });

        doc
          .moveTo(innerMargin + padding, tableTop + 15)
          .lineTo(pageWidth - innerMargin - padding, tableTop + 15)
          .stroke();

        const items = Array.isArray(data.items) ? data.items : [];

        let serviceCharge = 0;
        let yPosition = tableTop + 20;
        interface InvoiceItem {
          description?: string;
          quantity?: number;
          unitPrice?: number;
          total?: number;
        }

        items.forEach((item: InvoiceItem) => {
          let horizontalPosition: number = innerMargin + padding;
          doc
            .font('Helvetica-Bold')
            .fillColor(headerColor)
            .text(item.description || '', horizontalPosition, yPosition);
          horizontalPosition += columnWidths[0];

          doc
            .moveTo(horizontalPosition, yPosition - 5)
            .lineTo(horizontalPosition, yPosition + 15)
            .strokeColor(tableBorderColor)
            .stroke();

          doc.text('', horizontalPosition + 5, yPosition);
          horizontalPosition += columnWidths[1];

          doc
            .moveTo(horizontalPosition, yPosition - 5)
            .lineTo(horizontalPosition, yPosition + 15)
            .stroke(tableBorderColor);

          doc.text(
            String(item.quantity || 0),
            horizontalPosition + 5,
            yPosition
          );
          horizontalPosition += columnWidths[2];

          doc
            .moveTo(horizontalPosition, yPosition - 5)
            .lineTo(horizontalPosition, yPosition + 15)
            .stroke(tableBorderColor);

          doc.text(
            String((item.unitPrice || 0).toFixed(2)),
            horizontalPosition + 5,
            yPosition
          );
          horizontalPosition += columnWidths[3];

          doc
            .moveTo(horizontalPosition, yPosition - 5)
            .lineTo(horizontalPosition, yPosition + 15)
            .stroke(tableBorderColor);

          doc.text(
            String((item.total || 0).toFixed(2)),
            horizontalPosition + 5,
            yPosition
          );

          if (item.description === 'Service Charge') {
            serviceCharge = item.total || 0;
          }

          doc
            .moveTo(innerMargin + padding, yPosition + 15)
            .lineTo(pageWidth - innerMargin - padding, yPosition + 15)
            .stroke(headerColor);

          yPosition += 20;
        });

        for (let i = 0; i < 5; i++) {
          currentX = innerMargin + padding;

          doc
            .rect(
              innerMargin + padding,
              yPosition - 5,
              pageWidth - 2 * (innerMargin + padding),
              20
            )
            .fill(i % 2 === 0 ? '#D3D3D3' : '#FFFFFF');

          doc.fillColor(textColor).text('', currentX, yPosition);
          currentX += columnWidths[0];

          doc
            .moveTo(currentX, yPosition - 5)
            .lineTo(currentX, yPosition + 15)
            .stroke(tableBorderColor);

          doc.text('', currentX, yPosition);
          currentX += columnWidths[1];

          doc
            .moveTo(currentX, yPosition - 5)
            .lineTo(currentX, yPosition + 15)
            .stroke(tableBorderColor);
          doc.text('', currentX, yPosition);
          currentX += columnWidths[2];

          doc
            .moveTo(currentX, yPosition - 5)
            .lineTo(currentX, yPosition + 15)
            .stroke(tableBorderColor);

          doc.text('', currentX, yPosition);
          currentX += columnWidths[3];

          doc
            .moveTo(currentX, yPosition - 5)
            .lineTo(currentX, yPosition + 15)
            .stroke(tableBorderColor);

          doc.text('', currentX, yPosition);

          doc
            .moveTo(innerMargin + padding, yPosition + 15)
            .lineTo(pageWidth - innerMargin - padding, yPosition + 15)
            .stroke();

          yPosition += 20;
        }

        const remarksStartY = yPosition + 10;
        doc
          .fillColor(headerColor)
          .fontSize(8)
          .text(
            'Remarks / Payment Instructions',
            innerMargin + padding,
            remarksStartY
          );

        const subtotal: number = items.reduce(
          (sum: number, item: InvoiceItem) => sum + (item.total || 0),
          0
        );
        const cgst = serviceCharge * 1.18 - serviceCharge;
        const total = subtotal + cgst;

        yPosition += 10;

        const totalsTable = [
          ['SUBTOTAL', subtotal.toFixed(0)],
          ['DISCOUNT', '-'],
          ['SUBTOTAL LESS DISCOUNT', '-'],
          ['CGST', cgst.toFixed(0)],
          ['SGST', '-'],
          ['IGST', '-'],
          ['TOTAL', total.toFixed(0)],
          ['Balance Due (INR)', total.toFixed(0)],
        ];

        totalsTable.forEach(([label, value], index) => {
          const rowY = yPosition + index * 20;

          doc
            .rect(pageWidth - innerMargin - padding - 200, rowY - 5, 200, 20)
            .fill('#FFFFFF')
            .stroke();

          doc
            .fontSize(6)
            .fillColor(headerColor)
            .text(label, pageWidth - innerMargin - padding - 195, rowY)
            .fontSize(8)
            .fillColor(textColor)
            .text(value, pageWidth - innerMargin - padding - 60, rowY);

          doc
            .moveTo(pageWidth - innerMargin - padding - 200, rowY + 15)
            .lineTo(pageWidth - innerMargin - padding, rowY + 15)
            .stroke(headerColor);

          doc
            .moveTo(pageWidth - innerMargin - padding - 90, rowY - 5)
            .lineTo(pageWidth - innerMargin - padding - 90, rowY + 15)
            .stroke(headerColor);
        });

        doc
          .moveTo(pageWidth - innerMargin - padding - 200, yPosition - 5)
          .lineTo(
            pageWidth - innerMargin - padding - 200,
            yPosition + totalsTable.length * 20 - 5
          )
          .stroke(headerColor);

        doc
          .moveTo(pageWidth - innerMargin - padding, yPosition - 5)
          .lineTo(
            pageWidth - innerMargin - padding,
            yPosition + totalsTable.length * 20 - 5
          )
          .stroke(headerColor);

        doc
          .fillColor(headerColor)
          .fontSize(8)
          .text(
            'GST is charged on Karisjye\'s Service charge',
            innerMargin + padding,
            remarksStartY + 150
          );

        const footerYPosition = pageHeight - innerMargin - 30;
        doc
          .fillColor(textColor)
          .fontSize(8)
          .text(
            'NOTE : Document is digitally generated hence signature not required.',
            innerMargin + padding,
            footerYPosition,
            {
              align: 'center',
            }
          );

        doc.end();
      } catch (error) {
        reject(error);
      }
    });
  }

  async generateEstimatePDF(estimateData: any): Promise<Buffer> {
    // API to estimate invoice pdf for the particular booking to download in the frontend by the user
    const doc = new PDFDocument({
      size: 'A4',
      margin: 50,
    });

    const buffers: Buffer[] = [];
    doc.on('data', buffers.push.bind(buffers));

    const contentX = 60;
    const contentY = 50;
    let contentHeight = 0;

    doc.fontSize(16).text('ESTIMATE', { align: 'center' }).moveDown();
    contentHeight += 30;

    doc
      .moveTo(contentX, contentY + contentHeight - 10)
      .lineTo(540, contentY + contentHeight - 10)
      .stroke();

    const billingAddressBlockY = contentY + contentHeight + 70;
    const billingAddressBlockHeight = 20;
    const billingAddressBlockWidth = 240;

    doc
      .rect(
        contentX - 10,
        billingAddressBlockY,
        billingAddressBlockWidth,
        billingAddressBlockHeight
      )
      .strokeColor('#000')
      .lineWidth(1)
      .stroke();

    const karishyeBlockY = contentY + contentHeight;
    const karishyeBlockHeight = 120;
    const karishyeBlockWidth = 240;

    doc
      .rect(
        contentX - 10,
        karishyeBlockY - 10,
        karishyeBlockWidth,
        karishyeBlockHeight
      )
      .strokeColor('#000')
      .lineWidth(1)
      .stroke();

    doc
      .moveTo(contentX - 10, contentY + contentHeight + 70)
      .lineTo(contentX + 230, contentY + contentHeight + 70)
      .stroke();
    doc
      .moveTo(contentX - 10, contentY + contentHeight + 90)
      .lineTo(contentX + 230, contentY + contentHeight + 90)
      .stroke();

    doc
      .rect(contentX - 10, contentY + contentHeight + 70, 240, 20)
      .fillColor('#C1D8C3')
      .fill();

    doc
      .fillColor('#000')
      .fontSize(12)
      .font('Helvetica-Bold')
      .text('Billing Address', contentX, contentY + contentHeight + 75)
      .fontSize(10)
      .font('Helvetica-Bold')
      .text('KARISHYE PRIVATE LIMITED', contentX, contentY + contentHeight)
      .font('Helvetica')
      .text(
        'H-802, Aparna Cyberzon, Nallagandla,',
        contentX,
        contentY + contentHeight + 15
      )
      .font('Helvetica')
      .text('Hyderabad 500019', contentX, contentY + contentHeight + 30)
      .font('Helvetica-Bold')
      .text('GSTIN:', contentX + 15, contentY + contentHeight + 45)
      .font('Helvetica')
      .text('36AAJCK3293M1ZY', contentX + 50, contentY + contentHeight + 45)
      .font('Helvetica-Bold')
      .text('CIN:', contentX + 30, contentY + contentHeight + 60)
      .font('Helvetica')
      .text(
        'U72900TG2022PTC159676',
        contentX + 50,
        contentY + contentHeight + 60
      );
    contentHeight += 100;

    doc
      .fontSize(10)
      .font('Helvetica-Bold')
      .text(`Estimate No.: ${estimateData.estimateNo}`, 350, contentY + 30)
      .font('Helvetica-Bold')
      .text(`Estimate Date: ${estimateData.estimateDate}`, 350, contentY + 45)
      .text('M.No: 9985806208', 350, contentY + 60)
      .font('Helvetica-Bold')
      .text('WWW.KARISHYE.CO', 400, contentY + 90)
      .fontSize(12)
      .text('Shipping Address', 350, contentY + 105);
    contentHeight += 40;

    doc
      .moveTo(345, contentY + 100)
      .lineTo(550, contentY + 100)
      .stroke();
    doc
      .moveTo(345, contentY + 120)
      .lineTo(550, contentY + 120)
      .stroke();

    doc
      .rect(345, contentY + 100, 205, 20)
      .fillColor('#C1D8C3')
      .fill();

    doc
      .fontSize(12)
      .font('Helvetica-Bold')
      .fillColor('#000')
      .text('Shipping Address', 350, contentY + 105);

    contentHeight += 40;

    const blockX = 345;
    const blockY = contentY + 20;
    const blockWidth = 205;
    const blockHeight = 120;

    doc
      .rect(blockX, blockY, blockWidth, blockHeight)
      .strokeColor('#000')
      .lineWidth(1)
      .stroke();

    const tableTop = contentY + contentHeight + 20;
    const itemHeight = 25;
    let yPosition = tableTop;
    const tableHeaders = ['S.no.', 'PRODUCT', 'QTY', 'RATE'];
    const columnWidths = [80, 230, 80, 100];

    contentHeight = yPosition - contentY;

    doc
      .moveTo(contentX, tableTop - 5)
      .lineTo(contentX + columnWidths.reduce((a, b) => a + b, 0), tableTop - 5)
      .stroke();

    doc.fontSize(10).fillColor('#000');
    tableHeaders.forEach((header, i) => {
      doc.text(
        header,
        contentX + columnWidths.slice(0, i).reduce((a, b) => a + b, 0) + 5,
        yPosition
      );
    });

    doc
      .moveTo(contentX, yPosition + itemHeight - 5)
      .lineTo(
        contentX + columnWidths.reduce((a, b) => a + b, 0),
        yPosition + itemHeight - 5
      )
      .stroke();

    yPosition += itemHeight;
    contentHeight = yPosition - contentY;

    estimateData.items.forEach((item: any, index: number) => {
      doc
        .text(String(index + 1), contentX + 10, yPosition)
        .text(item?.name, contentX + 90, yPosition)
        .text(item?.qty, contentX + 320, yPosition)
        .text(item?.rate, contentX + 400, yPosition);

      doc
        .moveTo(contentX, yPosition + itemHeight - 5)
        .lineTo(
          contentX + columnWidths.reduce((a, b) => a + b, 0),
          yPosition + itemHeight - 5
        )
        .stroke();
      yPosition += itemHeight;
      contentHeight = yPosition - contentY;
    });

    let xPosition = contentX;
    columnWidths.forEach((width) => {
      doc
        .moveTo(xPosition, tableTop)
        .lineTo(xPosition, yPosition - 5)
        .stroke();
      xPosition += width;
    });

    doc
      .moveTo(xPosition, tableTop)
      .lineTo(xPosition, yPosition - 5)
      .stroke();

    const totalValue = estimateData.items.reduce(
      (total: number, item: any) => total + item.qty * item.rate,
      0
    );

    const totalValueYPosition = yPosition + 10;
    const totalValueHeight = 30;

    const totalValueX =
      contentX + columnWidths.slice(0, 3).reduce((a, b) => a + b, 0);
    doc
      .moveTo(totalValueX, tableTop - 5)
      .lineTo(totalValueX, totalValueYPosition + totalValueHeight)
      .stroke();
    doc
      .moveTo(totalValueX + columnWidths[3], tableTop - 5)
      .lineTo(
        totalValueX + columnWidths[3],
        totalValueYPosition + totalValueHeight
      )
      .stroke();

    doc
      .moveTo(totalValueX, totalValueYPosition + totalValueHeight)
      .lineTo(
        totalValueX + columnWidths[3],
        totalValueYPosition + totalValueHeight
      )
      .stroke();

    doc.fontSize(12).fillColor('#000');
    doc.text('Total Value', contentX + 400, totalValueYPosition + 10);
    doc.text(totalValue.toFixed(2), contentX + 410, totalValueYPosition - 10);

    contentHeight += 30;

    doc
      .rect(contentX - 10, contentY - 10, 500, contentHeight + 40)
      .strokeColor('#000')
      .lineWidth(1)
      .stroke();

    doc.end();

    return new Promise((resolve) => {
      doc.on('end', () => {
        resolve(Buffer.concat(buffers));
      });
    });
  }

  async registerUser(name: string, phoneNumber: string): Promise<string> {
    try {
      if (!name || typeof name !== 'string') {
        throw new Error('Name is required');
      }

      if (!phoneNumber) {
        throw new Error('Phone number is required');
      }

      const userRepository = AppDataSource.getRepository(Users);
      const { parsePhoneNumberFromString } = await import('libphonenumber-js');
      const phoneNumberObject = parsePhoneNumberFromString(phoneNumber);
      if (!phoneNumberObject || !phoneNumberObject.isValid()) {
        // used phonenumber with country code
        this.logger.error(`Invalid phone number format: ${phoneNumber}`);
        return 'Please enter a valid phone number with the correct country code.';
      }

      const formattedPhone = phoneNumberObject.number;

      const existingUser = await userRepository.findOne({
        where: { phoneNumber: formattedPhone },
      });

      if (existingUser) {
        return 'This phone number is already registered';
      }

      const newUser = userRepository.create({
        id: uuid(),
        name: name.trim(),
        phoneNumber: formattedPhone,
        createdAt: new Date(),
      });

      await userRepository.save(newUser);
      return 'Your form has been submitted.';
    } catch (error) {
      this.logger.error('Registration error:', error);
      throw new Error(`Failed to register user: ${error.message}`);
    }
  }
}
