import {
  Body,
  Controller,
  Get,
  Logger,
  Post,
  Query,
  Res,
} from '@nestjs/common';
import { Response } from 'express';
import { ApiBody, ApiCreatedResponse, ApiOperation } from '@nestjs/swagger';
import { APIFailedException } from 'src/common/exceptions';
import { CreateUsersDto, UserDetailsQuery } from './models/createUsers.dto';
import { ContactPayload } from './models/twilio.dto';
import { TwilioService } from './services/twilio/twilio.service';
import { UsersService } from './services/users/users.service';

interface UserResponse {
  status: string;
  data: CreateUsersDto;
}
interface UserDetailsResponse {
  status: string;
  data: CreateUsersDto;
}
@Controller('users')
export class UsersController {
  logger: Logger;

  constructor(
    private readonly usersService: UsersService,
    private twilio: TwilioService
  ) {
    this.logger = new Logger();
  }
  @Post('create-new') //added the route
  @ApiOperation({ summary: 'Create new user' })
  @ApiBody({ type: CreateUsersDto })
  @ApiCreatedResponse({ type: CreateUsersDto })
  async createUser(@Body() body: any): Promise<UserResponse> {
    try {
      this.logger.debug('API Triggered - createUser');
      const result = await this.usersService.createUser(body);

      if (!result) {
        throw new APIFailedException('Failed to create user');
      }

      return {
        status: 'success',
        data: result as unknown as CreateUsersDto,
      };
    } catch (error) {
      this.logger.error(error);
      throw new APIFailedException(error);
    }
  }

  @Get('')
  @ApiOperation({ summary: 'Get user details' })
  @ApiCreatedResponse({ type: CreateUsersDto })
  async getUserDetails(
    @Query() query: UserDetailsQuery
  ): Promise<UserDetailsResponse> {
    try {
      this.logger.debug('API Triggered - Get User');
      const result = await this.usersService.getUserDetails(query);

      if (!result) {
        throw new APIFailedException('User not found');
      }

      return {
        status: 'success',
        data: result as unknown as CreateUsersDto,
      };
    } catch (error) {
      this.logger.error('Failed to get user details:', error);
      throw new APIFailedException(error);
    }
  }

  @Post('contact-karishye')
  @ApiOperation({ summary: 'Contact Us API' })
  @ApiBody({ type: ContactPayload })
  // @ApiCreatedResponse({ type: any })
  contact(@Body() data: ContactPayload) {
    return this.usersService.contactKarishye(data);
  }

  @Post('generate-PDF')
  @ApiOperation({ summary: 'generating the PDF' })
  @ApiBody({ type: UserDetailsQuery })
  async generatePDF(@Body() data: any) {
    return this.usersService.generatePDF(data);
  }

  @Post('generate-invoice') // API to generate invoice PDF
  @ApiOperation({ summary: 'Generating the PDF' })
  async generateInvoice(@Body() invoiceData: any, @Res() res: Response) {
    try {
      const pdfBuffer = await this.usersService.generateInvoicePDF(invoiceData); //Generating the pdf

      res.set({
        'Content-Type': 'application/pdf',
        'Content-Disposition': 'attachment; filename=invoice.pdf',
        'Content-Length': pdfBuffer.length,
      });

      res.send(pdfBuffer);
    } catch (error) {
      res
        .status(500)
        .send({ message: 'Failed to generate PDF', error: error.message });
    }
  }

  @Post('generate-estimate') // API to generate estimate PDF
  @ApiOperation({ summary: 'Generating the Estimate PDF' })
  @ApiBody({ type: Object })
  async generateEstimatePDF(@Body() data: any, @Res() res: Response) {
    //Generating the pdf
    this.logger.log('data', data);
    const estimateData = {
      estimateNo: 'EST-00123',
      estimateDate: new Date().toISOString().split('T')[0],
      items: data.data.items,
    };
    this.logger.log('estimateData', estimateData);
    const pdfBuffer = await this.usersService.generateEstimatePDF(estimateData);

    res.set({
      'Content-Type': 'application/pdf',
      'Content-Disposition': 'attachment; filename=estimate.pdf',
      'Content-Length': pdfBuffer.length,
    });

    res.end(pdfBuffer);
  }

  @Post('register-user') // API to register a user with name and phone number
  async registerUser(
    @Body('name') name: string,
    @Body('phoneNumber') phoneNumber: string
  ): Promise<string> {
    this.logger.log('name', name);
    return this.usersService.registerUser(name, phoneNumber);
  }
}
